<?php

namespace Ems\Repository;

use Doctrine\ORM\EntityManager;
use Doctrine\DBAL\Connection;
use Teknisa\Model\Gestaopessoas\Repositories\GerVerificacao;

class EquipeRepository {
    /** @var Connection $connection */
    protected $connection;

    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
    }

    public function getMonitoramentoEquipeContrato(
        array $params
    ): array {

        $sql = <<<SQL
            SELECT VM.NRVINCULOM||'-'||VM.NMVINCULOM AS VINCULO,
                   VM.NRVINCULOM,
                   VM.NMVINCULOM,
				   TO_CHAR(VM.DTADMISSAOVINC,'DD/MM/YYYY') AS DTADMISSAOVINC,
				   VM.NRDIASCONTRATOEXP,
				   TO_CHAR( (VM.DTADMISSAOVINC + NVL(VM.NRDIASCONTRATOEXP, 0) - 1), 'DD/MM/YYYY') AS PRIMEIRA_PRORROGACAO,
				   VM.NRDIASCONTRATOEXPPRO,
                   TO_CHAR(VH.DTFIMCONTRDETERMIN,'DD/MM/YYYY') AS FIM_CONTRATO,
                   EH.NRESTRUTURAM,
                   EH.NMESTRUTURAH,
                   CP.CDCOMUNICAPARC,
				   TO_CHAR( (VM.DTADMISSAOVINC + NVL(VM.NRDIASCONTRATOEXP, 0) + NVL(VM.NRDIASCONTRATOEXPPRO, 0) - 1), 'DD/MM/YYYY') AS SEGUNDA_PRORROGACAO,
				   CEIL(VH.DTFIMCONTRDETERMIN - SYSDATE) AS DIAS_CONTRATO_VENCER
            FROM GPE_VINCULOM VM
            JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                        AND VH.NRVINCULOM = VM.NRVINCULOM
                        AND VH.DTMESCOMPETENC = (SELECT MAX(H1.DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                                    WHERE H1.NRORG = VH.NRORG
                                                    AND H1.NRVINCULOM = VH.NRVINCULOM
                                                    AND H1.DTMESCOMPETENC <= SYSDATE)
                LEFT JOIN ESTRUTURAH EH ON
                    EH.NRORG = VM.NRORG AND
                    EH.NRESTRUTURAM = VH.NRESTRUTGEREN AND
                    EH.DTMESCOMPETENC = (SELECT MAX(EHH.DTMESCOMPETENC) FROM ESTRUTURAH EHH WHERE
                                        EH.NRORG = EHH.NRORG AND
                                        EHH.NRESTRUTURAM = EH.NRESTRUTURAM
                                        AND EHH.DTMESCOMPETENC <= SYSDATE)
                LEFT JOIN COMUNICAPARC CP ON
                CP.NRORG = VM.NRORG AND
                CP.NRPARCNEGOCIO = EH.NRPARCRESPINF
                AND CP.CDFORMACOMU = '05'
            WHERE VM.DTRESCISAOVINC IS NULL
                AND VH.DTFIMCONTRDETERMIN IS NOT NULL
                /*AND VH.DTFIMCONTRDETERMIN BETWEEN SYSDATE AND SYSDATE + 10*/
                AND VM.NRORG = :P_NRORG
                AND (:P_NRESTRUTURAS IS NULL OR INSTR(:P_NRESTRUTURAS, ',' || VH.NRESTRUTGEREN || ',') > 0)
                
                /* BUSCA CONTRATOS A VENCER EM ATE 90 DIAS */
				AND SYSDATE BETWEEN (VM.DTADMISSAOVINC + (NVL (VM.NRDIASCONTRATOEXP, 0) + NVL(VM.NRDIASCONTRATOEXPPRO, 0) ) -91 ) AND (VM.DTADMISSAOVINC + (NVL (VM.NRDIASCONTRATOEXP, 0) + NVL(VM.NRDIASCONTRATOEXPPRO, 0) ) - 1 )
            ORDER BY VH.DTFIMCONTRDETERMIN
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $result = $std->fetchAll();

        return $result;
    }

    public function getMonitoramentoEquipeLicenca(
        array $params
    ): array {

        $sql = <<<SQL
            SELECT VM.NRVINCULOM||'-'||VM.NMVINCULOM AS VINCULO,
                TO_CHAR(SF.DTFIMSITUFUNC,'DD/MM/YYYY') AS DATA_FINAL,
                EH.NRESTRUTURAM,
                EH.NMESTRUTURAH,
                CP.CDCOMUNICAPARC,
                VM.NRVINCULOM,
                VM.NMVINCULOM
            FROM GPE_VINCULOM VM
            JOIN GPE_VINCULOH VH ON
                VH.NRORG = VM.NRORG AND
                VH.NRVINCULOM = VM.NRVINCULOM AND
                VH.DTMESCOMPETENC = (SELECT MAX(VHH.DTMESCOMPETENC) FROM GPE_VINCULOH VHH WHERE
                                        VH.NRORG = VHH.NRORG AND
                                        VHH.NRVINCULOM = VH.NRVINCULOM
                                        AND VHH.DTMESCOMPETENC <= SYSDATE)
            LEFT JOIN ESTRUTURAH EH ON
                    EH.NRORG = VM.NRORG AND
                    EH.NRESTRUTURAM = VH.NRESTRUTGEREN AND
                    EH.DTMESCOMPETENC = (SELECT MAX(EHH.DTMESCOMPETENC) FROM ESTRUTURAH EHH WHERE
                                        EH.NRORG = EHH.NRORG AND
                                        EHH.NRESTRUTURAM = EH.NRESTRUTURAM
                                        AND EHH.DTMESCOMPETENC <= SYSDATE)
            LEFT JOIN COMUNICAPARC CP ON
                CP.NRORG = VM.NRORG AND
                CP.NRPARCNEGOCIO = EH.NRPARCRESPINF
                AND CP.CDFORMACOMU = '05'
            JOIN GPE_ALTESITUFUNC SF ON SF.NRORG = VM.NRORG
                                    AND SF.NRVINCULOM = VM.NRVINCULOM
            WHERE VM.DTRESCISAOVINC IS NULL
                AND SF.NRMOTIVOAFASTA = 3
                AND SF.DTFIMSITUFUNC BETWEEN SYSDATE-30 AND SYSDATE+10
                AND VM.NRORG = :P_NRORG
                AND (
					:P_NRESTRUTURAS IS NULL
					OR (INSTR(:P_NRESTRUTURAS, ',' || VH.NRESTRUTGEREN || ',')) > 0 )
            ORDER BY SF.DTFIMSITUFUNC, VM.NMVINCULOM
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $result = $std->fetchAll();

        return $result;
    }

    public function getMonitoramentoEquipeBancoHoras(array $params): array
    {
        $sql = <<<SQL
            SELECT
                    NRVINCULOM,
                    NMVINCULOM,
                    --NVL(TO_CHAR(SALDO_A_FOLGAR_NORMAL),         0) SALDO_A_FOLGAR_NORMAL,
                    NVL(TO_CHAR(SALDO_FOLGAR_PER_BHR_NORM),     0) SALDO_FOLGAR_PER_BHR_NORM,
                    NVL(TO_CHAR(SALDO_FOLGAR_ACR_PER_BHR_NORM), 0) SALDO_FOLGAR_ACR_PER_BHR_NORM
                  FROM (
                      SELECT * FROM (
                          SELECT NRSITUFREQUENCIA,
                                 (ROUND(QTREFAPURSITUFREQ, 2)) QTREFAPURSITUFREQ,
                                 VM.NMVINCULOM,
                                 VM.NRVINCULOM,
                                 NRPERIODOAPURACAO,
                                 (SELECT NVL(ROUND(QTREFAPURSITUFREQ, 7), '0000')
                                     FROM (SELECT QTREFAPURSITUFREQ
                                             FROM PTO_RESUMOSITUFREQ
                                            WHERE NRVINCULOM        = VM.NRVINCULOM
                                              AND NRPERIODOAPURACAO = :P_NRPERIODOAPURACAO
                                              AND NRORG             = :P_NRORG
                                              AND NRSITUFREQUENCIA  = 18020
                                         ORDER BY DTAPURACAOSITUFREQ DESC)
                                    WHERE ROWNUM = 1) AS SALDO_FOLGAR_PER_BHR_NORM,
                                  (SELECT NVL(ROUND(QTREFAPURSITUFREQ, 7), '0000')
                                     FROM (SELECT QTREFAPURSITUFREQ
                                             FROM PTO_RESUMOSITUFREQ
                                            WHERE NRVINCULOM        = VM.NRVINCULOM
                                              AND NRPERIODOAPURACAO = :P_NRPERIODOAPURACAO
                                              AND NRORG             = :P_NRORG
                                              AND NRSITUFREQUENCIA  = 18420
                                         ORDER BY DTAPURACAOSITUFREQ DESC)
                                    WHERE ROWNUM = 1) AS SALDO_FOLGAR_ACR_PER_BHR_NORM
                            FROM PTO_RESUMOSITUFREQ
                            JOIN GPE_VINCULOM VM
                              ON VM.NRVINCULOM = PTO_RESUMOSITUFREQ.NRVINCULOM
                             AND VM.NRORG      = PTO_RESUMOSITUFREQ.NRORG
                           WHERE PTO_RESUMOSITUFREQ.NRPERIODOAPURACAO = :P_NRPERIODOAPURACAO
                             AND PTO_RESUMOSITUFREQ.NRORG             = :P_NRORG
                             AND EXISTS (SELECT 1
                                           FROM GPE_MOVIMENTACAO MOV
                                           JOIN ESTRUTURAM EM
                                             ON EM.NRESTRUTURAM = MOV.NRESTRUTURAM
                                            AND EM.NRORG        = MOV.NRORG
                                          WHERE MOV.NRORG          = VM.NRORG
                                            AND MOV.NRVINCULOM     = VM.NRVINCULOM
                                            AND MOV.NRTIPOTRANSFER = 2 /* movimentação gerencial */
											AND (
													:P_NRESTRUTURAS IS NULL
													OR (INSTR(:P_NRESTRUTURAS, ',' || MOV.NRESTRUTURAM || ',')) > 0 )
                                            AND MOV.DTINIMOVIMENT = (SELECT MAX(DTINIMOVIMENT)
                                                                        FROM GPE_MOVIMENTACAO
                                                                       WHERE NRORG           = MOV.NRORG
                                                                         AND NRVINCULOM      = MOV.NRVINCULOM
                                                                         AND NRTIPOESTRUTURA = MOV.NRTIPOESTRUTURA
                                                                         AND DTINIMOVIMENT  <= TO_DATE(:P_DTFINALAPURACAO, 'DD/MM/YYYY')))
                      ) PTO_RESUMOSITUFREQ
                  ) PIVOT (SUM(QTREFAPURSITUFREQ) FOR NRSITUFREQUENCIA IN (18000 SALDO_A_FOLGAR_NORMAL))
                /* SALDOS NEGATIVOS E POSITIVOS SÃO MOSTRADOS */
                /*WHERE TO_NUMBER(SALDO_FOLGAR_ACR_PER_BHR_NORM) > 0*/
                ORDER BY TO_NUMBER(SALDO_FOLGAR_ACR_PER_BHR_NORM) DESC
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

		$result = $std->fetchAll();

        return $result;
    }

    public function getMonitoramentoEquipeAtrasos($params) {
		$sql = <<<SQL
		SELECT
			NRVINCULOM,
			NMVINCULOM,
			NVL(TO_CHAR(ATRASOS_DIURNOS), 0) ATRASOS_DIURNOS
			FROM (
				SELECT * FROM (
					SELECT NRSITUFREQUENCIA,
							(ROUND(QTREFAPURSITUFREQ, 2)) QTREFAPURSITUFREQ,
							VM.NMVINCULOM,
							VM.NRVINCULOM,
							NRPERIODOAPURACAO
					FROM PTO_RESUMOSITUFREQ
					JOIN GPE_VINCULOM VM
						ON VM.NRVINCULOM = PTO_RESUMOSITUFREQ.NRVINCULOM
						AND VM.NRORG      = PTO_RESUMOSITUFREQ.NRORG
					WHERE PTO_RESUMOSITUFREQ.NRPERIODOAPURACAO = :P_NRPERIODOAPURACAO
						AND PTO_RESUMOSITUFREQ.NRORG             = :P_NRORG
						AND EXISTS (SELECT 1
									FROM GPE_MOVIMENTACAO MOV
									JOIN ESTRUTURAM EM
										ON EM.NRESTRUTURAM = MOV.NRESTRUTURAM
									AND EM.NRORG        = MOV.NRORG
									WHERE MOV.NRORG          = VM.NRORG
									AND MOV.NRVINCULOM     = VM.NRVINCULOM
									AND MOV.NRTIPOTRANSFER = 2 /* movimentação gerencial */
									AND (
										:P_NRESTRUTURAS IS NULL
										OR (INSTR(:P_NRESTRUTURAS, ',' ||MOV.NRESTRUTURAM ||',')) > 0 )
									AND MOV.DTINIMOVIMENT = (SELECT MAX(DTINIMOVIMENT)
																FROM GPE_MOVIMENTACAO
																WHERE NRORG           = MOV.NRORG
																	AND NRVINCULOM      = MOV.NRVINCULOM
																	AND NRTIPOESTRUTURA = MOV.NRTIPOESTRUTURA
																	AND DTINIMOVIMENT  <= TO_DATE(:P_DTFINALAPURACAO, 'DD/MM/YYYY')))
				) PTO_RESUMOSITUFREQ
			) PIVOT (SUM(QTREFAPURSITUFREQ) FOR NRSITUFREQUENCIA IN (10100 ATRASOS_DIURNOS))
		WHERE TO_NUMBER(ATRASOS_DIURNOS) > 0
		ORDER BY TO_NUMBER(ATRASOS_DIURNOS) DESC
SQL;

		$query = $this->connection->prepare($sql);
		$query->execute($params);

		$result = $query->fetchAll();

		return $result;
	}

	public function getMonitoramentoEquipeFaltas($params) {
		$sql = <<<SQL
			SELECT
				NRVINCULOM,
				NMVINCULOM,
				NVL(TO_CHAR(FALTAS),         0) QTD_FALTAS,
				NVL(TO_CHAR(FALTAS_DIURNAS), 0) FALTAS_HORAS
			FROM (
				SELECT * FROM (
					SELECT NRSITUFREQUENCIA,
							(ROUND(QTREFAPURSITUFREQ, 2)) QTREFAPURSITUFREQ,
							VM.NMVINCULOM,
							VM.NRVINCULOM,
							NRPERIODOAPURACAO,
							(SELECT COUNT(QTREFAPURSITUFREQ)
								FROM PTO_RESUMOSITUFREQ
							WHERE NRVINCULOM        = VM.NRVINCULOM
								AND NRPERIODOAPURACAO = :P_NRPERIODOAPURACAO
								AND NRORG             = :P_NRORG
								AND NRSITUFREQUENCIA  = 10500) AS FALTAS
						FROM PTO_RESUMOSITUFREQ
						JOIN GPE_VINCULOM VM
						ON VM.NRVINCULOM = PTO_RESUMOSITUFREQ.NRVINCULOM
						AND VM.NRORG      = PTO_RESUMOSITUFREQ.NRORG
						WHERE PTO_RESUMOSITUFREQ.NRPERIODOAPURACAO = :P_NRPERIODOAPURACAO
						AND PTO_RESUMOSITUFREQ.NRORG             = :P_NRORG
						AND EXISTS (SELECT 1
										FROM GPE_MOVIMENTACAO MOV
										JOIN ESTRUTURAM EM
											ON EM.NRESTRUTURAM = MOV.NRESTRUTURAM
										AND EM.NRORG        = MOV.NRORG
										WHERE MOV.NRORG          = VM.NRORG
										AND MOV.NRVINCULOM     = VM.NRVINCULOM
										AND MOV.NRTIPOTRANSFER = 2 /* movimentação gerencial */
										AND (
											:P_NRESTRUTURAS IS NULL
											OR (INSTR(:P_NRESTRUTURAS, ',' || MOV.NRESTRUTURAM || ',')) > 0 )
										AND MOV.DTINIMOVIMENT = (SELECT MAX(DTINIMOVIMENT)
																	FROM GPE_MOVIMENTACAO
																	WHERE NRORG           = MOV.NRORG
																		AND NRVINCULOM      = MOV.NRVINCULOM
																		AND NRTIPOESTRUTURA = MOV.NRTIPOESTRUTURA
																		AND DTINIMOVIMENT  <= TO_DATE(:P_DTFINALAPURACAO, 'DD/MM/YYYY')))
				) PTO_RESUMOSITUFREQ
			) PIVOT (SUM(QTREFAPURSITUFREQ) FOR NRSITUFREQUENCIA IN (10500 FALTAS_DIURNAS))
			WHERE TO_NUMBER(FALTAS) > 0
			ORDER BY TO_NUMBER(QTD_FALTAS) DESC
SQL;

		$query = $this->connection->prepare($sql);
		$query->execute($params);

		$result = $query->fetchAll();

		return $result;
	}

	public function getMonitoramentoEquipeHorasExtras($params) {
		$sql = <<<SQL
			SELECT
				NRVINCULOM,
				NMVINCULOM,
				NVL(TO_CHAR(EX_DIA_NORMAL_SEG),    0) EX_DIA_NORMAL_SEG,
				NVL(TO_CHAR(EX_DIA_NORMAL_NP_SEG), 0) EX_DIA_NORMAL_NP_SEG
			FROM (
				SELECT * FROM (
					SELECT NRSITUFREQUENCIA,
							(ROUND(QTREFAPURSITUFREQ, 2)) QTREFAPURSITUFREQ,
							VM.NMVINCULOM,
							VM.NRVINCULOM,
							NRPERIODOAPURACAO
					FROM PTO_RESUMOSITUFREQ
					JOIN GPE_VINCULOM VM
						ON VM.NRVINCULOM = PTO_RESUMOSITUFREQ.NRVINCULOM
						AND VM.NRORG      = PTO_RESUMOSITUFREQ.NRORG
					WHERE PTO_RESUMOSITUFREQ.NRPERIODOAPURACAO = :P_NRPERIODOAPURACAO
						AND PTO_RESUMOSITUFREQ.NRORG             = :P_NRORG
						AND EXISTS (SELECT 1
									FROM GPE_MOVIMENTACAO MOV
									JOIN ESTRUTURAM EM
										ON EM.NRESTRUTURAM = MOV.NRESTRUTURAM
									AND EM.NRORG        = MOV.NRORG
									WHERE MOV.NRORG          = VM.NRORG
									AND MOV.NRVINCULOM     = VM.NRVINCULOM
									AND MOV.NRTIPOTRANSFER = 2 /* movimentação gerencial */
									AND (
										:P_NRESTRUTURAS IS NULL
										OR (INSTR(:P_NRESTRUTURAS, ',' || MOV.NRESTRUTURAM || ',')) > 0 )
									AND MOV.DTINIMOVIMENT = (SELECT MAX(DTINIMOVIMENT)
																FROM GPE_MOVIMENTACAO
																WHERE NRORG           = MOV.NRORG
																	AND NRVINCULOM      = MOV.NRVINCULOM
																	AND NRTIPOESTRUTURA = MOV.NRTIPOESTRUTURA
																	AND DTINIMOVIMENT  <= TO_DATE(:P_DTFINALAPURACAO, 'DD/MM/YYYY')))
				) PTO_RESUMOSITUFREQ
			) PIVOT (SUM(QTREFAPURSITUFREQ) FOR NRSITUFREQUENCIA IN (2100 EX_DIA_NORMAL_NP_SEG, 2300 EX_DIA_NORMAL_SEG))
		WHERE TO_NUMBER(EX_DIA_NORMAL_SEG) > 0
		ORDER BY TO_NUMBER(EX_DIA_NORMAL_SEG) DESC
SQL;

		$query = $this->connection->prepare($sql);
		$query->execute($params);

		$result = $query->fetchAll();

		return $result;
	}

	public function getMonitoramentoEquipeAtestados($params) {
		$sql = <<<SQL
			SELECT
				NRVINCULO,
				NMVINCULOM,
				ATESTADO_DIA,
				ATESTADO_COMPARECIMENTO,
				(ATESTADO_DIA + ATESTADO_COMPARECIMENTO) TOTAL_ATESTADOS
			FROM
			(SELECT
				VM.NRVINCULOM,
				VM.NRVINCULOM NRVINCULO,
				VM.NMVINCULOM,
				NRMOTIVOOCORFREQ
			FROM PTO_PROGOCORRENCIA PROG
			JOIN GPE_VINCULOM VM
				ON VM.NRVINCULOM = PROG.NRVINCULOM
				AND VM.NRORG      = PROG.NRORG
			WHERE PROG.NRORG        = :P_NRORG
				AND EXISTS (SELECT 1
							FROM GPE_MOVIMENTACAO MOV
							JOIN ESTRUTURAM EM
								ON EM.NRESTRUTURAM = MOV.NRESTRUTURAM
								AND EM.NRORG        = MOV.NRORG
							WHERE MOV.NRORG          = VM.NRORG
								AND MOV.NRVINCULOM     = VM.NRVINCULOM
								AND MOV.NRTIPOTRANSFER = 2 /* movimentação gerencial */
								AND (
									:P_NRESTRUTURAS IS NULL
									OR (INSTR(:P_NRESTRUTURAS, ',' || MOV.NRESTRUTURAM || ',')) > 0 )
								AND MOV.DTINIMOVIMENT = (SELECT MAX(DTINIMOVIMENT)
														FROM GPE_MOVIMENTACAO
														WHERE NRORG           = MOV.NRORG
															AND NRVINCULOM      = MOV.NRVINCULOM
															AND NRTIPOESTRUTURA = MOV.NRTIPOESTRUTURA
															AND DTINIMOVIMENT  <= TO_DATE(:P_DTFINALAPURACAO, 'DD/MM/YYYY')))
				AND PROG.DTINICIOPROGOCOR BETWEEN TO_DATE(:P_DTINICIALAPURACAO, 'DD/MM/YYYY') AND TO_DATE(:P_DTFINALAPURACAO, 'DD/MM/YYYY')
				AND PROG.NRTIPOPROGOCORRENCIA IN (5, 17)
				AND PROG.IDTIPODIACALENDARIO IS NULL
				AND PROG.NRMOTIVOOCORFREQ IN (5, 23) /* 5 - Atestado Doença de 1 a 15 dias, 23 - Atestado de Comparecimento */) PROG
				PIVOT (COUNT(NRVINCULOM) FOR NRMOTIVOOCORFREQ IN (5 ATESTADO_DIA, 23 ATESTADO_COMPARECIMENTO))
			WHERE TO_NUMBER(ATESTADO_DIA + ATESTADO_COMPARECIMENTO) > 0
			ORDER BY TO_NUMBER(TOTAL_ATESTADOS) DESC
SQL;

		$query = $this->connection->prepare($sql);
		$query->execute($params);

		$result = $query->fetchAll();

		return $result;
	}

	public function getMonitoramentoEquipeFerias($params) {
        $sql = <<<SQL
            SELECT
                VM.NRVINCULOM,
                VM.NMVINCULOM,
                DECODE(FE.IDCONTROLFERIAS, 'ADQUIRIDO', 'Adquirido', 'GOZADO_PARCIAL', 'Gozado Parcial') IDCONTROLFERIAS,
                TO_CHAR(FE.DTINIAQUISICAO, 'DD/MM/YYYY') DTINIAQUISICAO,
                TO_CHAR(FE.DTFIMAQUISICAO, 'DD/MM/YYYY') DTFIMAQUISICAO,
				NVL(1 + (GZ.DTFIMGOZOFERIAS) - (GZ.DTINIGOZOFERIAS), 0) AS DIAS_GOZO,
				TO_CHAR(ADD_MONTHS(FE.DTINIAQUISICAO, 23) + NVL((GZ.DTFIMGOZOFERIAS) - (GZ.DTINIGOZOFERIAS), 0) - 1, 'DD/MM/YYYY') AS DATA_LIMITE,
				FE.QTDIASMAXFE,
				FE.QTDIASMAXABON,
                TO_CHAR(FE.DTINIAQUISICAO, 'DD/MM/YYYY') || ' a ' || TO_CHAR(FE.DTFIMAQUISICAO, 'DD/MM/YYYY') AS PERIODO_AQUISICAO
                FROM GPE_VINCULOM VM
				JOIN GPE_VINCULOH   VH ON VH.NRORG = VM.NRORG
                            AND VH.NRVINCULOM = VM.NRVINCULOM
                            AND VH.DTMESCOMPETENC = (
					SELECT
						MAX(H1.DTMESCOMPETENC) FROM
						GPE_VINCULOH H1
					WHERE
						H1.NRORG = VH.NRORG
						AND H1.NRVINCULOM = VH.NRVINCULOM
						AND H1.DTMESCOMPETENC <= SYSDATE
				)
				LEFT JOIN ESTRUTURAH     EH ON EH.NRORG = VM.NRORG
										AND EH.NRESTRUTURAM = VH.NRESTRUTGEREN AND EH.DTMESCOMPETENC = (
					SELECT
						MAX(EHH.DTMESCOMPETENC)
					FROM
						ESTRUTURAH EHH
					WHERE
						EH.NRORG = EHH.NRORG
						AND EHH.NRESTRUTURAM = EH.NRESTRUTURAM
						AND EHH.DTMESCOMPETENC <= SYSDATE
				)
				LEFT JOIN COMUNICAPARC   CP ON CP.NRORG = VM.NRORG
											AND CP.NRPARCNEGOCIO = EH.NRPARCRESPINF
											AND CP.CDFORMACOMU = '05'
                JOIN FPA_FERIAS FE ON FE.NRORG = VM.NRORG
                                AND FE.NRVINCULOM = VM.NRVINCULOM
								JOIN FPA_TIPOFERIASH T ON (T.NRORG = 0 OR T.NRORG = FE.NRORG)
  								AND T.NRTIPOFERIAS = FE.NRTIPOFERIAS
				LEFT JOIN FPA_GOZOFERIAS GZ ON GZ.NRORG = FE.NRORG
					AND GZ.NRFERIAS = FE.NRFERIAS
				JOIN FPA_TIPOFERIASH T ON ( T.NRORG = 0 OR T.NRORG = FE.NRORG )
					AND T.NRTIPOFERIAS = FE.NRTIPOFERIAS
                WHERE ADD_MONTHS(FE.DTINIAQUISICAO, 23) BETWEEN SYSDATE AND ADD_MONTHS(SYSDATE, 3)
                AND FE.IDCONTROLFERIAS IN ('ADQUIRIDO', 'GOZADO_PARCIAL')
                AND VM.DTRESCISAOVINC IS NULL
                AND VM.NRORG = :P_NRORG
                AND EXISTS (SELECT 1
                                FROM GPE_MOVIMENTACAO MOV
                                JOIN ESTRUTURAM EM
                                ON EM.NRESTRUTURAM = MOV.NRESTRUTURAM
                                AND EM.NRORG        = MOV.NRORG
                            WHERE MOV.NRORG          = VM.NRORG
                                AND MOV.NRVINCULOM     = VM.NRVINCULOM
                                AND MOV.NRTIPOTRANSFER = 2 /* movimentação gerencial */
                                AND (
									:P_NRESTRUTURAS IS NULL
									OR (INSTR(:P_NRESTRUTURAS, ',' || MOV.NRESTRUTURAM || ',')) > 0 )
                                AND MOV.DTINIMOVIMENT = (SELECT MAX(DTINIMOVIMENT)
                                                        FROM GPE_MOVIMENTACAO
                                                        WHERE NRORG           = MOV.NRORG
                                                            AND NRVINCULOM      = MOV.NRVINCULOM
                                                            AND NRTIPOESTRUTURA = MOV.NRTIPOESTRUTURA
                                                            AND DTINIMOVIMENT  <= TO_DATE(:P_DTFINALAPURACAO, 'DD/MM/YYYY')))
                ORDER BY ADD_MONTHS(FE.DTINIAQUISICAO, 23)
SQL;

		$query = $this->connection->prepare($sql);
		$query->execute($params);

		$result = $query->fetchAll();

		return $result;
	}
	
	public function getMonitoramentoEquipeAfastamentos($params) {
        $sql = <<<SQL
			SELECT
				VM.NRVINCULOM,
				VM.NMVINCULOM,
				TO_CHAR( SF.DTINISITUFUNC,'DD/MM/YYYY' ) AS DATA_INICIO,
				TO_CHAR( SF.DTFIMSITUFUNC,'DD/MM/YYYY' ) AS DATA_TERMINO,
				CEIL( (CASE WHEN DTFIMSITUFUNC IS NULL THEN SYSDATE ELSE DTFIMSITUFUNC END) - SF.DTINISITUFUNC + 1 ) AS DIAS_AFASTAMENTO,
				GPE_SITUFUNCH.IDAFASTSITUFUNC AS SITU_AFAST,
				GPE_SITUFUNCH.NMSITUFUNCH
				FROM GPE_VINCULOM VM
				JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
					AND VH.NRVINCULOM = VM.NRVINCULOM
					AND VH.DTMESCOMPETENC = (
						SELECT
							MAX( VHH.DTMESCOMPETENC )
						FROM
							GPE_VINCULOH VHH
						WHERE
							VH.NRORG = VHH.NRORG
							AND VHH.NRVINCULOM = VH.NRVINCULOM
							AND VHH.DTMESCOMPETENC <= SYSDATE
						)
			LEFT JOIN ESTRUTURAH EH ON EH.NRORG = VM.NRORG
				AND EH.NRESTRUTURAM = VH.NRESTRUTGEREN
				AND EH.DTMESCOMPETENC = (
					SELECT
					MAX( EHH.DTMESCOMPETENC )
					FROM
					ESTRUTURAH EHH
					WHERE
						EH.NRORG = EHH.NRORG
						AND EHH.NRESTRUTURAM = EH.NRESTRUTURAM
						AND EHH.DTMESCOMPETENC <= SYSDATE
						)
			LEFT JOIN COMUNICAPARC CP ON CP.NRORG = VM.NRORG
				AND CP.NRPARCNEGOCIO = EH.NRPARCRESPINF
				AND CP.CDFORMACOMU = '05'
			JOIN GPE_ALTESITUFUNC SF ON SF.NRORG = VM.NRORG
				AND SF.NRVINCULOM = VM.NRVINCULOM
			INNER JOIN GPE_SITUFUNCH ON
				GPE_SITUFUNCH.NRSITUFUNCM = SF.NRSITUFUNCM
				AND GPE_SITUFUNCH.DTEFETIVA = (SELECT MAX(H1.DTEFETIVA) FROM GPE_SITUFUNCH H1
											WHERE H1.NRSITUFUNCM = GPE_SITUFUNCH.NRSITUFUNCM)
			LEFT JOIN GPE_MOTIVOAFASTA AF ON SF.NRMOTIVOAFASTA = AF.NRMOTIVOAFASTA
			WHERE
			VM.DTRESCISAOVINC IS NULL
			AND SF.NRMOTIVOAFASTA NOT IN (5, 12, 15, 16, 100042)
			AND SF.DTFIMSITUFUNC BETWEEN SYSDATE-30 AND SYSDATE+10
			AND CEIL( (CASE WHEN DTFIMSITUFUNC IS NULL THEN SYSDATE ELSE DTFIMSITUFUNC END) - SF.DTINISITUFUNC + 1 ) > 15
			AND GPE_SITUFUNCH.IDAFASTSITUFUNC = 'S'
			AND VM.NRORG = :P_NRORG
			AND (
				:P_NRESTRUTURAS IS NULL
			OR (INSTR(:P_NRESTRUTURAS, ',' || VH.NRESTRUTGEREN || ',')) > 0 )

			ORDER BY
			VM.NMVINCULOM,
			AF.NMMOTIVOAFASTA,
			AF.NRMOTIVOAFASTA

SQL;

		$query = $this->connection->prepare($sql);
		$query->execute($params);

		$result = $query->fetchAll();

		return $result;
	}
	
	public function getMonitoramentoEquipeUltimasFerias($params) {
        $sql = <<<SQL
	        SELECT
			    NRVINCULOM,
			    NMVINCULOM,
			    DTRESCISAOVINC,
			    NRFERIAS,
			    NRTIPOFERIAS,
			    TO_CHAR(DTINIAQUISICAO, 'DD/MM/YYYY') DTINIAQUISICAO,
			    TO_CHAR(DTFIMAQUISICAO, 'DD/MM/YYYY') DTFIMAQUISICAO,
			    NVL(QTDIASMAXFE_FERIAS, QTDIASMAXFE_TIPOFERIAS) QTDIASMAXFE,
			    DECODE(IDCONTROLFERIAS,
			        'ADQUIRIDO',		'Adquirido',
			        'GOZADO_PARCIAL',	'Gozado Parcial',
			        'GOZADO_INTEGRAL',	'Gozado Integral',
			        'ABERTO',			'Aberto',
			        'PERDIDO',			'Perdido',
			        IDCONTROLFERIAS
			    ) IDCONTROLFERIAS,
			    NRGOZOFERIAS,
			    TO_CHAR(DTINIGOZOFERIAS, 'DD/MM/YYYY') DTINIGOZOFERIAS,
			    TO_CHAR(DTFIMGOZOFERIAS, 'DD/MM/YYYY') DTFIMGOZOFERIAS,
			    TOTAL_GOZO,
			    DTRETORNOFERIAS,
			    TOTAL_ABONO,
			   (NVL(QTDIASMAXFE_FERIAS, QTDIASMAXFE_TIPOFERIAS) - TOTAL_GOZO - TOTAL_ABONO) DIAS_RESTANTE_GOZO,
			   DECODE((
                        SELECT COUNT(1)
                        FROM FPA_FERIAS FE
                        JOIN FPA_GOZOFERIAS GF ON GF.NRORG = FE.NRORG AND GF.NRFERIAS = FE.NRFERIAS
                        WHERE FE.NRORG = ULT_FERIAS.NRORG AND FE.NRVINCULOM = ULT_FERIAS.NRVINCULOM
                            AND  (:P_DTINICIALAPURACAO <= GF.DTFIMGOZOFERIAS AND :P_DTFINALAPURACAO >= GF.DTINIGOZOFERIAS)
                    ),
			    0, 'N', 'S') VINC_EM_GOZO
			FROM (
			    SELECT
			            VM.NRORG,
			            VM.NRVINCULOM,
			            VM.NMVINCULOM,
			            VM.DTRESCISAOVINC,
			            FE.NRFERIAS,
			            FE.NRTIPOFERIAS,
			            FE.DTINIAQUISICAO,
			            FE.DTFIMAQUISICAO,
			            FE.QTDIASMAXFE QTDIASMAXFE_FERIAS,
			            T.QTDIASMAXFE QTDIASMAXFE_TIPOFERIAS,
			            FE.IDCONTROLFERIAS,
			            
			            MAX(GF.NRGOZOFERIAS) NRGOZOFERIAS,
			            MAX(GF.DTINIGOZOFERIAS) DTINIGOZOFERIAS,
			            MAX(GF.DTFIMGOZOFERIAS) DTFIMGOZOFERIAS,
			            NVL(SUM(1 + (GF.DTFIMGOZOFERIAS) - (GF.DTINIGOZOFERIAS)), 0) TOTAL_GOZO,
			            MAX(GF.DTRETORNOFERIAS) DTRETORNOFERIAS,
			            NVL(SUM(GF.QTDIASABONOFERI), 0)  TOTAL_ABONO
			        FROM GPE_VINCULOM VM
			        JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG AND VH.NRVINCULOM = VM.NRVINCULOM AND VH.DTMESCOMPETENC = (
			            SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH WHERE NRORG = VH.NRORG AND NRVINCULOM = VH.NRVINCULOM AND DTMESCOMPETENC <= :P_DTFINALAPURACAO
			        )
			        LEFT JOIN FPA_FERIAS FE ON FE.NRORG = VM.NRORG AND FE.NRVINCULOM = VM.NRVINCULOM
			            AND FE.DTINIAQUISICAO = (
			            SELECT MAX(DTINIAQUISICAO) FROM FPA_FERIAS FE2 WHERE FE2.NRORG = FE.NRORG AND FE2.NRVINCULOM = FE.NRVINCULOM AND FE2.DTINIAQUISICAO <= :P_DTFINALAPURACAO
			                AND EXISTS (
			                    SELECT 1
			                    FROM FPA_GOZOFERIAS
			                    WHERE NRORG = FE2.NRORG
			                        AND NRFERIAS = FE2.NRFERIAS
			                )
			        )
			        LEFT JOIN FPA_TIPOFERIASH T ON (T.NRORG = 0 OR T.NRORG = FE.NRORG) AND T.NRTIPOFERIAS = FE.NRTIPOFERIAS
			        LEFT JOIN FPA_GOZOFERIAS GF ON GF.NRORG = VM.NRORG AND GF.NRFERIAS = FE.NRFERIAS
			        WHERE VM.NRORG = :P_NRORG
			            AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= LAST_DAY(:P_DTFINALAPURACAO))
			            AND (:P_NRESTRUTURAS IS NULL OR INSTR(:P_NRESTRUTURAS, ',' || VH.NRESTRUTGEREN || ',') > 0)
			        GROUP BY VM.NRORG, VM.NRVINCULOM, VM.NMVINCULOM, VM.DTRESCISAOVINC, FE.NRFERIAS, FE.NRTIPOFERIAS, FE.DTINIAQUISICAO, FE.DTFIMAQUISICAO, FE.QTDIASMAXFE, T.QTDIASMAXFE, FE.IDCONTROLFERIAS
			        ORDER BY DTINIGOZOFERIAS DESC NULLS LAST, NMVINCULOM
			) ULT_FERIAS
SQL;

		$query = $this->connection->prepare($sql);
		$query->execute($params);

		$result = $query->fetchAll();

		return $result;
	}

}