<?php

namespace Ems\Repository;

use Doctrine\ORM\EntityManager;
use Ems\Util\ExpectedException;

class PeriodoRepository {
    protected $connection;

    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
    }

    public function getPeriodo($nrorg, $nrperiodopuracao) {
        $sql = <<<SQL
            SELECT
                *
            FROM
                PTO_PERIODOAPURACAO PPA
            WHERE
                PPA.NRORG = :P_NRORG
                AND PPA.NRPERIODOAPURACAO = :P_NRPERIODOAPURACAO
SQL;

        $params = array(
            'P_NRORG' => $nrorg,
            'P_NRPERIODOAPURACAO' => $nrperiodopuracao
        );

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $periodo = $std->fetch();

        return $periodo;
    }

    public function getPeriodoAtual($params) {
        $sql = <<<SQL
            SELECT  PPA.*
            FROM    PTO_PERIODOAPURACAO PPA,
                    PTO_TIPOPERIODOAPURACAO TPA
            WHERE   PPA.NRORG = :P_NRORG
                AND TPA.NRTIPOVINCULOM = :P_NRTIPOVINCULOM
                --BUSCA O PERIODO PELO DIA ATUAL, SEM CONTAR A HORA
                AND TRUNC(SYSDATE) BETWEEN PPA.DTINICIALAPURACAO AND PPA.DTFINALAPURACAO
                AND PPA.NRTIPOPERIODOAPURACAO = TPA.NRTIPOPERIODOAPURACAO
                AND PPA.NRORG = TPA.NRORG
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $periodo = $std->fetch();

        if (!$periodo) {
            $nrTipoVinculo = $params['P_NRTIPOVINCULOM'];

            throw new ExpectedException("Não há período para a data atual! (Tipo de vínculo: $nrTipoVinculo)");
        }

        return $periodo;
    }

    public function getPeriodoIntervalo($nrorg, $periodo){
        $query = <<<SQL
            SELECT TO_CHAR(PA.DTINICIALAPURACAO, 'DD/MM/YYYY') DTINICIALAPURACAO,
                   TO_CHAR(PA.DTFINALAPURACAO, 'DD/MM/YYYY') DTFINALAPURACAO,
                   (NVL( SUBSTR(PO.HRTEMPODESCANSODIARIO, 1, 2) + ( SUBSTR(PO.HRTEMPODESCANSODIARIO, 4, 2) / 60 ), 11)) HRTEMPODESCANSODIARIO
            FROM PTO_PERIODOAPURACAO PA
            JOIN GPE_PARAMORG PO ON PO.NRORG = PA.NRORG
            WHERE PA.NRORG             = :P_NRORG
              AND PA.NRPERIODOAPURACAO = :P_PERIODO
SQL;

        $params = array('P_NRORG'   => $nrorg,
                        'P_PERIODO' => $periodo);

        $result = \Setforms2\Shared\Registry::getEntityManagerConnection()->fetchAll($query, $params);

        return $result;
    }

    public function getPeriodoDemonstrativo(
        int $nrOrg,
        int $nrPeriodo
    ): array
    {
        $params = [
            'P_NRORG'     => $nrOrg,
            'P_NRPERIODO' => $nrPeriodo
        ];

        $sql = <<<SQL
                SELECT PPA.nrperiodoapuracao,
                    To_char(PPA.dtinicialapuracao, 'DD/MM/YYYY')
                    || ' - '
                    || To_char(PPA.dtfinalapuracao, 'DD/MM/YYYY')
                    || ' ('
                    || PPA.idperiodoapuracao
                    || ')' AS DSPERIODOAPURACAO,
                    PPA.nrorg,
                    TO_CHAR(PPA.dtfinalapuracao, 'DD/MM/YYYY') AS DTFINALAPURACAO,
                    TO_CHAR(PPA.dtinicialapuracao, 'DD/MM/YYYY') AS DTINICIALAPURACAO,
                    PPA.nrtipoperiodoapuracao,
                    PPA.idperiodoapuracao
                FROM   pto_periodoapuracao PPA,
                    pto_tipoperiodoapuracao TPA
                WHERE  PPA.nrorg = :P_NRORG
                    AND PPA.nrperiodoapuracao = :P_NRPERIODO
                    AND PPA.nrtipoperiodoapuracao = TPA.nrtipoperiodoapuracao
                    AND PPA.nrorg = TPA.nrorg
                ORDER  BY PPA.dtinicialapuracao DESC
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $periodos = $std->fetchAll();

        return $periodos;
    }
    
    // Criar um periodo service pra padronizar
    public function getDtmescompetencByPeriodo(
        string $nrorg,
        string $nrperiodoapuracao   = null,
        string $dtinicialapuracao   = null,
        string $dtfinalapuracao     = null
    ) {
        // Passar isso pra um 'periodoServices' pra poder parametrizar
        // é melhor deixar o repository pra query sem validações
        if (empty($nrperiodoapuracao) && empty($dtinicialapuracao) && empty($dtfinalapuracao)) {
            $params = array(
                'P_NRORG'             => $nrorg,
                'P_NRTIPOVINCULOM'    => 1
            );
            
            $periodoAtual = $this->getPeriodoAtual($params);
            if (!empty($periodoAtual)) {
                $nrperiodoapuracao = $periodoAtual['NRPERIODOAPURACAO'];
            }
        }
        $sql = <<<SQL
            SELECT
                TO_CHAR(PPA.DTFINALAPURACAO, 'DD/MM/YYYY') DTFINALAPURACAO
            FROM
                PTO_PERIODOAPURACAO PPA
            WHERE
                PPA.NRORG = :P_NRORG
                AND (PPA.NRPERIODOAPURACAO   = :P_NRPERIODOAPURACAO OR :P_NRPERIODOAPURACAO IS NULL)
                AND (PPA.DTINICIALAPURACAO   = TO_DATE(:P_DTINICIALAPURACAO, 'DD/MM/YYYY') OR :P_DTINICIALAPURACAO IS NULL)
                AND (PPA.DTFINALAPURACAO     = TO_DATE(:P_DTFINALAPURACAO, 'DD/MM/YYYY')   OR :P_DTFINALAPURACAO   IS NULL)
SQL;

        $params = array(
            'P_NRORG'               => $nrorg,
            'P_NRPERIODOAPURACAO'   => $nrperiodoapuracao,
            'P_DTINICIALAPURACAO'   => $dtinicialapuracao,
            'P_DTFINALAPURACAO'     => $dtfinalapuracao
        );

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $periodo = $std->fetch();
        
        if (!empty($periodo)) {
            return $periodo['DTFINALAPURACAO'];
        }

        return $periodo;
    }
    
    public function findPeriodoByDia(
                $nrorg,
                $nrvinculom,
        string  $dia
    ) {
        
        $sql = <<<SQL
            SELECT
                PPA.*
            FROM
                PTO_PERIODOAPURACAO       PPA,
                PTO_TIPOPERIODOAPURACAO   TPA
                
                JOIN GPE_VINCULOM VM ON VM.NRVINCULOM = :P_NRVINCULOM AND VM.NRORG = :P_NRORG
            WHERE
                PPA.NRORG = :P_NRORG
                AND TPA.NRTIPOVINCULOM = VM.NRTIPOVINCULOM
                AND TO_DATE(:P_DATE, 'DD/MM/YYYY') BETWEEN PPA.DTINICIALAPURACAO AND PPA.DTFINALAPURACAO
                AND PPA.NRTIPOPERIODOAPURACAO = TPA.NRTIPOPERIODOAPURACAO
                AND PPA.NRORG = TPA.NRORG
SQL;

        $params = array(
            'P_NRORG'       => $nrorg,
            'P_NRVINCULOM'  => $nrvinculom,
            'P_DATE'        => $dia
        );

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $periodo = $std->fetch();

        return $periodo;
    }
    
    public function getDataInicialUltPeriodoAberto(
                $nrorg,
                $nrvinculom
        // string  $dia
    ) {
        
        $sql = <<<SQL
            SELECT
                --*
                TO_CHAR(MIN(ppa.dtinicialapuracao), 'DD/MM/YYYY') dtinicialapuracao
            FROM
                PTO_PERIODOAPURACAO       PPA,
                PTO_TIPOPERIODOAPURACAO   TPA
                
                JOIN GPE_VINCULOM VM ON VM.NRVINCULOM = :P_NRVINCULOM AND VM.NRORG = :P_NRORG
            WHERE
                PPA.NRORG = :P_NRORG
                AND TPA.NRTIPOVINCULOM = VM.NRTIPOVINCULOM
                --AND SYSDATE BETWEEN PPA.DTINICIALAPURACAO AND PPA.DTFINALAPURACAO
                AND PPA.IDPERIODOAPURACAO = 'ABERTO'
                
                
                AND PPA.NRTIPOPERIODOAPURACAO = TPA.NRTIPOPERIODOAPURACAO
                AND PPA.NRORG = TPA.NRORG
SQL;

        $params = array(
            'P_NRORG'       => $nrorg,
            'P_NRVINCULOM'  => $nrvinculom,
        );

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $periodo = $std->fetch();

        return $periodo;
    }
}